@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2018 The Project Lombok Authors.
+ * Copyright (C) 2009-2019 The Project Lombok Authors.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,6 @@ package lombok.javac.apt;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.Writer;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
 import java.net.URL;
@@ -37,21 +36,16 @@ import java.util.Set;
 import java.util.SortedSet;
 
 import javax.annotation.processing.AbstractProcessor;
-import javax.annotation.processing.Filer;
 import javax.annotation.processing.Messager;
 import javax.annotation.processing.ProcessingEnvironment;
 import javax.annotation.processing.RoundEnvironment;
 import javax.annotation.processing.SupportedAnnotationTypes;
 import javax.lang.model.SourceVersion;
 import javax.lang.model.element.Element;
+import javax.lang.model.element.QualifiedNameable;
 import javax.lang.model.element.TypeElement;
 import javax.tools.Diagnostic.Kind;
 import javax.tools.JavaFileManager;
-import javax.tools.JavaFileObject;
-
-import lombok.Lombok;
-import lombok.core.DiagnosticsReceiver;
-import lombok.javac.JavacTransformer;
 
 import com.sun.source.util.TreePath;
 import com.sun.source.util.Trees;
@@ -62,6 +56,12 @@ import com.sun.tools.javac.processing.JavacProcessingEnvironment;
 import com.sun.tools.javac.tree.JCTree.JCCompilationUnit;
 import com.sun.tools.javac.util.Context;
 
+import lombok.Lombok;
+import lombok.core.CleanupRegistry;
+import lombok.core.DiagnosticsReceiver;
+import lombok.javac.JavacTransformer;
+import lombok.permit.Permit;
+
 /**
  * This Annotation Processor is the standard injection mechanism for lombok-enabling the javac compiler.
  * 
@@ -70,6 +70,7 @@ import com.sun.tools.javac.util.Context;
  */
 @SupportedAnnotationTypes("*")
 public class LombokProcessor extends AbstractProcessor {
+
 	private ProcessingEnvironment processingEnv;
 	private JavacProcessingEnvironment javacProcessingEnv;
 	private JavacFiler javacFiler;
@@ -111,10 +112,7 @@ public class LombokProcessor extends AbstractProcessor {
 	
 	private static final Field getFieldAccessor(String typeName, String fieldName) {
 		try {
-			Class<?> c = Class.forName(typeName);
-			Field f = c.getDeclaredField(fieldName);
-			f.setAccessible(true);
-			return f;
+			return Permit.getField(Class.forName(typeName), fieldName);
 		} catch (ClassNotFoundException e) {
 			return null;
 		} catch (NoSuchFieldException e) {
@@ -155,25 +153,25 @@ public class LombokProcessor extends AbstractProcessor {
 		Context context = javacProcessingEnv.getContext();
 		disablePartialReparseInNetBeansEditor(context);
 		try {
-			Method keyMethod = Context.class.getDeclaredMethod("key", Class.class);
-			keyMethod.setAccessible(true);
+			Method keyMethod = Permit.getMethod(Context.class, "key", Class.class);
 			Object key = keyMethod.invoke(context, JavaFileManager.class);
-			Field htField = Context.class.getDeclaredField("ht");
-			htField.setAccessible(true);
+			Field htField = Permit.getField(Context.class, "ht");
 			@SuppressWarnings("unchecked")
 			Map<Object,Object> ht = (Map<Object,Object>) htField.get(context);
 			final JavaFileManager originalFiler = (JavaFileManager) ht.get(key);
 			if (!(originalFiler instanceof InterceptingJavaFileManager)) {
 				final Messager messager = processingEnv.getMessager();
 				DiagnosticsReceiver receiver = new MessagerDiagnosticsReceiver(messager);
-
+				
 				JavaFileManager newFilerManager = new InterceptingJavaFileManager(originalFiler, receiver);
 				ht.put(key, newFilerManager);
-				Field filerFileManagerField = JavacFiler.class.getDeclaredField("fileManager");
-				filerFileManagerField.setAccessible(true);
+				Field filerFileManagerField = Permit.getField(JavacFiler.class, "fileManager");
 				filerFileManagerField.set(javacFiler, newFilerManager);
-
-				replaceFileManagerJdk9(context, newFilerManager);
+				
+				if (lombok.javac.Javac.getJavaCompilerVersion() > 8
+						&& !lombok.javac.handlers.JavacHandlerUtil.inNetbeansCompileOnSave(context)) {
+					replaceFileManagerJdk9(context, newFilerManager);
+				}
 			}
 		} catch (Exception e) {
 			throw Lombok.sneakyThrow(e);
@@ -182,20 +180,17 @@ public class LombokProcessor extends AbstractProcessor {
 
 	private void replaceFileManagerJdk9(Context context, JavaFileManager newFiler) {
 		try {
-			JavaCompiler compiler = (JavaCompiler) JavaCompiler.class.getDeclaredMethod("instance", Context.class).invoke(null, context);
+			JavaCompiler compiler = (JavaCompiler) Permit.getMethod(JavaCompiler.class, "instance", Context.class).invoke(null, context);
 			try {
-				Field fileManagerField = JavaCompiler.class.getDeclaredField("fileManager");
-				fileManagerField.setAccessible(true);
+				Field fileManagerField = Permit.getField(JavaCompiler.class, "fileManager");
 				fileManagerField.set(compiler, newFiler);
 			}
 			catch (Exception e) {}
 			
 			try {
-				Field writerField = JavaCompiler.class.getDeclaredField("writer");
-				writerField.setAccessible(true);
+				Field writerField = Permit.getField(JavaCompiler.class, "writer");
 				ClassWriter writer = (ClassWriter) writerField.get(compiler);
-				Field fileManagerField = ClassWriter.class.getDeclaredField("fileManager");
-				fileManagerField.setAccessible(true);
+				Field fileManagerField = Permit.getField(ClassWriter.class, "fileManager");
 				fileManagerField.set(writer, newFiler);
 			}
 			catch (Exception e) {}
@@ -206,8 +201,7 @@ public class LombokProcessor extends AbstractProcessor {
 	
 	private void forceMultipleRoundsInNetBeansEditor() {
 		try {
-			Field f = JavacProcessingEnvironment.class.getDeclaredField("isBackgroundCompilation");
-			f.setAccessible(true);
+			Field f = Permit.getField(JavacProcessingEnvironment.class, "isBackgroundCompilation");
 			f.set(javacProcessingEnv, true);
 		} catch (NoSuchFieldException e) {
 			// only NetBeans has it
@@ -219,14 +213,12 @@ public class LombokProcessor extends AbstractProcessor {
 	private void disablePartialReparseInNetBeansEditor(Context context) {
 		try {
 			Class<?> cancelServiceClass = Class.forName("com.sun.tools.javac.util.CancelService");
-			Method cancelServiceInstace = cancelServiceClass.getDeclaredMethod("instance", Context.class);
+			Method cancelServiceInstace = Permit.getMethod(cancelServiceClass, "instance", Context.class);
 			Object cancelService = cancelServiceInstace.invoke(null, context);
 			if (cancelService == null) return;
-			Field parserField = cancelService.getClass().getDeclaredField("parser");
-			parserField.setAccessible(true);
+			Field parserField = Permit.getField(cancelService.getClass(), "parser");
 			Object parser = parserField.get(cancelService);
-			Field supportsReparseField = parser.getClass().getDeclaredField("supportsReparse");
-			supportsReparseField.setAccessible(true);
+			Field supportsReparseField = Permit.getField(parser.getClass(), "supportsReparse");
 			supportsReparseField.set(parser, false);
 		} catch (ClassNotFoundException e) {
 			// only NetBeans has it
@@ -280,8 +272,7 @@ public class LombokProcessor extends AbstractProcessor {
 	
 	private void stopJavacProcessingEnvironmentFromClosingOurClassloader() {
 		try {
-			Field f = JavacProcessingEnvironment.class.getDeclaredField("processorClassLoader");
-			f.setAccessible(true);
+			Field f = Permit.getField(JavacProcessingEnvironment.class, "processorClassLoader");
 			ClassLoader unwrapped = (ClassLoader) f.get(javacProcessingEnv);
 			if (unwrapped == null) return;
 			ClassLoader wrapped = wrapClassLoader(unwrapped);
@@ -296,11 +287,15 @@ public class LombokProcessor extends AbstractProcessor {
 	private final IdentityHashMap<JCCompilationUnit, Long> roots = new IdentityHashMap<JCCompilationUnit, Long>();
 	private long[] priorityLevels;
 	private Set<Long> priorityLevelsRequiringResolutionReset;
+	private CleanupRegistry cleanup = new CleanupRegistry();
 	
 	/** {@inheritDoc} */
 	@Override public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {
 		if (lombokDisabled) return false;
-		if (roundEnv.processingOver()) return false;
+		if (roundEnv.processingOver()) {
+			cleanup.run();
+			return false;
+		}
 		
 		// We have: A sorted set of all priority levels: 'priorityLevels'
 		
@@ -326,7 +321,7 @@ public class LombokProcessor extends AbstractProcessor {
 					if (prioOfCu == null || prioOfCu != prio) continue;
 					cusForThisRound.add(entry.getKey());
 				}
-				transformer.transform(prio, javacProcessingEnv.getContext(), cusForThisRound);
+				transformer.transform(prio, javacProcessingEnv.getContext(), cusForThisRound, cleanup);
 			}
 			
 			// Step 3: Push up all CUs to the next level. Set level to null if there is no next level.
@@ -365,25 +360,18 @@ public class LombokProcessor extends AbstractProcessor {
 	private void forceNewRound(String randomModuleName, JavacFiler filer) {
 		if (!filer.newFiles()) {
 			try {
-				String name = "lombok.dummy.ForceNewRound" + (dummyCount++);
-				if (randomModuleName != null) name = randomModuleName + "/" + name;
-				JavaFileObject dummy = filer.createSourceFile(name);
-				Writer w = dummy.openWriter();
-				w.close();
+				filer.getGeneratedSourceNames().add("lombok.dummy.ForceNewRound" + (dummyCount++));
 			} catch (Exception e) {
 				e.printStackTrace();
 				processingEnv.getMessager().printMessage(Kind.WARNING,
-						"Can't force a new processing round. Lombok won't work.");
+					"Can't force a new processing round. Lombok won't work.");
 			}
 		}
 	}
 	
 	private String getModuleNameFor(Element element) {
 		while (element != null) {
-			if (element.getKind().name().equals("MODULE")) {
-				String n = element.getSimpleName().toString().trim();
-				return n.isEmpty() ? null : n;
-			}
+			if (element.getKind().name().equals("MODULE")) return ModuleNameOracle.getModuleName(element);
 			Element n = element.getEnclosingElement();
 			if (n == element) return null;
 			element = n;
@@ -391,8 +379,26 @@ public class LombokProcessor extends AbstractProcessor {
 		return null;
 	}
 	
+	// QualifiedNameable is a java7 thing, so to remain compatible with java6, shove this into an inner class to avoid the ClassNotFoundError.
+	private static class ModuleNameOracle {
+		static String getModuleName(Element element) {
+			if (!(element instanceof QualifiedNameable)) return null;
+			String name = ((QualifiedNameable) element).getQualifiedName().toString().trim();
+			return name.isEmpty() ? null : name;
+		}
+	}
+	
 	private JCCompilationUnit toUnit(Element element) {
-		TreePath path = trees == null ? null : trees.getPath(element);
+		TreePath path = null;
+		if (trees != null) {
+			try {
+				path = trees.getPath(element);
+			} catch (NullPointerException ignore) {
+				// Happens if a package-info.java dowsn't conatin a package declaration.
+				// https://github.com/rzwitserloot/lombok/issues/2184
+				// We can safely ignore those, since they do not need any processing
+			}
+		}
 		if (path == null) return null;
 		
 		return (JCCompilationUnit) path.getCompilationUnit();
@@ -404,47 +410,51 @@ public class LombokProcessor extends AbstractProcessor {
 	@Override public SourceVersion getSupportedSourceVersion() {
 		return SourceVersion.latest();
 	}
-
+	
 	/**
 	 * This class casts the given processing environment to a JavacProcessingEnvironment. In case of
 	 * gradle incremental compilation, the delegate ProcessingEnvironment of the gradle wrapper is returned.
 	 */
-	public JavacProcessingEnvironment getJavacProcessingEnvironment(ProcessingEnvironment procEnv) {
-		final Class<?> procEnvClass = procEnv.getClass();
-		if (procEnv.getClass().getName().equals("org.gradle.api.internal.tasks.compile.processing.IncrementalProcessingEnvironment")) {
+	public JavacProcessingEnvironment getJavacProcessingEnvironment(Object procEnv) {
+		if (procEnv instanceof JavacProcessingEnvironment) return (JavacProcessingEnvironment) procEnv;
+		
+		// try to find a "delegate" field in the object, and use this to try to obtain a JavacProcessingEnvironment
+		for (Class<?> procEnvClass = procEnv.getClass(); procEnvClass != null; procEnvClass = procEnvClass.getSuperclass()) {
 			try {
-				Field field = procEnvClass.getDeclaredField("delegate");
-				field.setAccessible(true);
-				Object delegate = field.get(procEnv);
-				return (JavacProcessingEnvironment) delegate;
+				return getJavacProcessingEnvironment(tryGetDelegateField(procEnvClass, procEnv));
 			} catch (final Exception e) {
-				e.printStackTrace();
-				procEnv.getMessager().printMessage(Kind.WARNING,
-						"Can't get the delegate of the gradle IncrementalProcessingEnvironment. Lombok won't work.");
+				// delegate field was not found, try on superclass
 			}
 		}
-		return (JavacProcessingEnvironment) procEnv;
+		
+		processingEnv.getMessager().printMessage(Kind.WARNING,
+			"Can't get the delegate of the gradle IncrementalProcessingEnvironment. Lombok won't work.");
+		return null;
 	}
 
 	/**
-	 * This class casts the given filer to a JavacFiler. In case of
-	 * gradle incremental compilation, the delegate Filer of the gradle wrapper is returned.
+	 * This class returns the given filer as a JavacFiler. In case the filer is no
+	 * JavacFiler (e.g. the Gradle IncrementalFiler), its "delegate" field is used to get the JavacFiler
+	 * (directly or through a delegate field again)
 	 */
-	public JavacFiler getJavacFiler(Filer filer) {
-		final Class<?> filerSuperClass = filer.getClass().getSuperclass();
-		if (filerSuperClass.getName().equals("org.gradle.api.internal.tasks.compile.processing.IncrementalFiler")) {
+	public JavacFiler getJavacFiler(Object filer) {
+		if (filer instanceof JavacFiler) return (JavacFiler) filer;
+		
+		// try to find a "delegate" field in the object, and use this to check for a JavacFiler
+		for (Class<?> filerClass = filer.getClass(); filerClass != null; filerClass = filerClass.getSuperclass()) {
 			try {
-				Field field = filerSuperClass.getDeclaredField("delegate");
-				field.setAccessible(true);
-				Object delegate = field.get(filer);
-				return (JavacFiler) delegate;
+				return getJavacFiler(tryGetDelegateField(filerClass, filer));
 			} catch (final Exception e) {
-				e.printStackTrace();
-				processingEnv.getMessager().printMessage(Kind.WARNING,
-						"Can't get the delegate of the gradle IncrementalFiler. Lombok won't work.");
+				// delegate field was not found, try on superclass
 			}
 		}
-		return (JavacFiler) filer;
+		
+		processingEnv.getMessager().printMessage(Kind.WARNING,
+			"Can't get a JavacFiler from " + filer.getClass().getName() + ". Lombok won't work.");
+		return null;
 	}
 
+	private Object tryGetDelegateField(Class<?> delegateClass, Object instance) throws Exception {
+		return Permit.getField(delegateClass, "delegate").get(instance);
+	}
 }
