@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 The Project Lombok Authors.
+ * Copyright (C) 2016-2020 The Project Lombok Authors.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,7 @@ import java.io.Writer;
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.util.Arrays;
 import java.util.HashMap;
 import java.util.Map;
 
@@ -93,6 +94,7 @@ import com.sun.tools.javac.util.Position;
 
 import lombok.javac.CommentInfo;
 import lombok.javac.PackageName;
+import lombok.permit.Permit;
 import lombok.javac.CommentInfo.EndConnection;
 import lombok.javac.CommentInfo.StartConnection;
 import lombok.javac.JavacTreeMaker.TreeTag;
@@ -152,6 +154,7 @@ public class PrettyPrinter extends JCTree.Visitor {
 	private final Writer out;
 	private final JCCompilationUnit compilationUnit;
 	private List<CommentInfo> comments;
+	private final int[] textBlockStarts;
 	private final FormatPreferences formatPreferences;
 	
 	private final Map<JCTree, String> docComments;
@@ -159,9 +162,10 @@ public class PrettyPrinter extends JCTree.Visitor {
 	private int indent = 0;
 	
 	@SuppressWarnings({"unchecked", "rawtypes"})
-	public PrettyPrinter(Writer out, JCCompilationUnit cu, List<CommentInfo> comments, FormatPreferences preferences) {
+	public PrettyPrinter(Writer out, JCCompilationUnit cu, List<CommentInfo> comments, int[] textBlockStarts, FormatPreferences preferences) {
 		this.out = out;
 		this.comments = comments;
+		this.textBlockStarts = textBlockStarts;
 		this.compilationUnit = cu;
 		this.formatPreferences = preferences;
 		
@@ -257,6 +261,7 @@ public class PrettyPrinter extends JCTree.Visitor {
 				JCMethodInvocation inv = (JCMethodInvocation) expr;
 				if (!inv.typeargs.isEmpty() || !inv.args.isEmpty()) return false;
 				if (!(inv.meth instanceof JCIdent)) return false;
+				if (tree.pos != expr.pos) return false; // Explicit super call
 				return ((JCIdent) inv.meth).name.toString().equals("super");
 			}
 		}
@@ -376,6 +381,9 @@ public class PrettyPrinter extends JCTree.Visitor {
 	private int dims(JCExpression vartype) {
 		if (vartype instanceof JCArrayTypeTree) {
 			return 1 + dims(((JCArrayTypeTree) vartype).elemtype);
+		} else if (isJcAnnotatedType(vartype)) {
+			JCTree underlyingType = readObject(vartype, "underlyingType", (JCTree) null);
+			if (underlyingType instanceof JCArrayTypeTree) return 1 + dims (((JCArrayTypeTree) underlyingType).elemtype);
 		}
 		
 		return 0;
@@ -483,6 +491,19 @@ public class PrettyPrinter extends JCTree.Visitor {
 	}
 	
 	@Override public void visitImport(JCImport tree) {
+		if (tree.qualid instanceof JCFieldAccess) {
+			JCFieldAccess fa = ((JCFieldAccess) tree.qualid);
+			if (fa.name.length() == 1 && fa.name.contentEquals("*")) {
+				if (fa.selected instanceof JCFieldAccess) {
+					JCFieldAccess lombokExperimental = (JCFieldAccess) fa.selected;
+					if (lombokExperimental.name.contentEquals("experimental") && lombokExperimental.selected instanceof JCIdent && ((JCIdent) lombokExperimental.selected).name.contentEquals("lombok")) {
+						// do not ever print lombok.experimental.*.
+						return;
+					}
+				}
+			}
+		}
+		
 		aPrint("import ");
 		if (tree.staticImport) print("static ");
 		print(tree.qualid);
@@ -623,13 +644,33 @@ public class PrettyPrinter extends JCTree.Visitor {
 		printVarDef0(tree);
 	}
 	
+	private boolean innermostArrayBracketsAreVarargs = false;
 	private void printVarDef0(JCVariableDecl tree) {
 		boolean varargs = (tree.mods.flags & VARARGS) != 0;
-		if (varargs && tree.vartype instanceof JCArrayTypeTree) {
-			print(((JCArrayTypeTree) tree.vartype).elemtype);
-			print("...");
-		} else {
-			print(tree.vartype);
+		
+		/* story time!
+		 
+		 in 'new int[5][6];', the 5 is the outermost and the 6 is the innermost: That means: 5 int arrays, each capable of containing 6 elements.
+		 But that's actually a crazy way to read it; you'd think that in FOO[], you should interpret that as 'an array of FOO', but that's not correct;
+		 if FOO is for example 'int[]', it's: "Modify the component type of FOO to be an array of whatever it was before.. unless FOO isn't an array, in which case,
+		 this is an array of FOO". Which is weird.
+		 
+		 This is particularly poignant with vargs. In: "int[]... x", the ... are actually the _INNER_ type even though varargs by definition is a modification of
+		 how to interpret the outer. The JLS just sort of lets that be: To indicate varargs, replace the lexically last [] with dots even though that's the wrong
+		 [] to modify!
+		 
+		 This becomes an utter shambles when annotations-on-arrays become involved. The annotation on the INNER most type is to be placed right before the ...;
+		 and because of that, we have to do crazy stuff with this innermostArrayBracketsAreVarargs flag.
+		 */
+		try {
+			innermostArrayBracketsAreVarargs = varargs;
+			if (tree.vartype == null || tree.vartype.pos == -1) {
+				print("var");
+			} else {
+				print(tree.vartype);
+			}
+		} finally {
+			innermostArrayBracketsAreVarargs = false;
 		}
 		print(" ");
 		print(tree.name);
@@ -667,7 +708,10 @@ public class PrettyPrinter extends JCTree.Visitor {
 	@Override public void visitTypeApply(JCTypeApply tree) {
 		print(tree.clazz);
 		print("<");
+		boolean temp = innermostArrayBracketsAreVarargs;
+		innermostArrayBracketsAreVarargs = false;
 		print(tree.arguments, ", ");
+		innermostArrayBracketsAreVarargs = temp;
 		print(">");
 	}
 	
@@ -699,7 +743,37 @@ public class PrettyPrinter extends JCTree.Visitor {
 		}
 		else if (CTC_BOOLEAN.equals(typeTag)) print(((Number)tree.value).intValue() == 1 ? "true" : "false");
 		else if (CTC_BOT.equals(typeTag)) print("null");
-		else print("\"" + quoteChars(tree.value.toString()) + "\"");
+		else {
+			if (Arrays.binarySearch(textBlockStarts, tree.pos) < 0) {
+				print("\"" + quoteChars(tree.value.toString()) + "\"");
+			} else {
+				printTextBlock(tree.value.toString());
+			}
+		}
+	}
+	
+	private void printTextBlock(String s) {
+		println("\"\"\"");
+		needsAlign = true;
+		indent++;
+		StringBuilder sb = new StringBuilder();
+		boolean lineStart = true;
+		for (int i = 0; i < s.length(); i++) {
+			char c = s.charAt(i);
+			if (c != ' ' && c != '\t') lineStart = false;
+			if (c == '\n') {
+				println(sb);
+				sb.setLength(0);
+				needsAlign = true;
+				lineStart = true;
+				continue;
+			}
+			if (c == '\t' && lineStart) sb.append("\t");
+			else sb.append(quoteChar(s.charAt(i)));
+		}
+		print(sb);
+		print("\"\"\"");
+		indent--;
 	}
 	
 	@Override public void visitMethodDef(JCMethodDecl tree) {
@@ -773,10 +847,7 @@ public class PrettyPrinter extends JCTree.Visitor {
 	}
 	
 	@Override public void visitTypeArray(JCArrayTypeTree tree) {
-		JCTree elem = tree.elemtype;
-		while (elem instanceof JCWildcard) elem = ((JCWildcard) elem).inner;
-		print(elem);
-		print("[]");
+		printTypeArray0(tree);
 	}
 	
 	@Override public void visitNewArray(JCNewArray tree) {
@@ -1000,7 +1071,11 @@ public class PrettyPrinter extends JCTree.Visitor {
 	@Override public void visitTypeTest(JCInstanceOf tree) {
 		print(tree.expr);
 		print(" instanceof ");
-		print(tree.clazz);
+		
+		/** With java14, instead of a type (field 'clazz'), there's now a pattern (field 'pattern', of type JCTree, which is either a JCPattern for new-style instanceof, or if not it's the same as what 'clazz' held) */
+		JCTree c = readObject(tree, "clazz", null); // JDK-13
+		if (c == null) c = readObject(tree, "pattern", null); // JDK14+
+		print(c);
 	}
 	
 	@Override public void visitTypeCast(JCTypeCast tree) {
@@ -1024,9 +1099,17 @@ public class PrettyPrinter extends JCTree.Visitor {
 	
 	@Override public void visitBreak(JCBreak tree) {
 		aPrint("break");
-		if (tree.label != null) {
+		
+		JCExpression value = readObject(tree, "value", null); // JDK 12+
+		if (value != null) {
 			print(" ");
-			print(tree.label);
+			print(value);
+		} else {
+			Name label = readObject(tree, "label", null);
+			if (label != null) {
+				print(" ");
+				print(label);
+			}
 		}
 		println(";", tree);
 	}
@@ -1208,16 +1291,46 @@ public class PrettyPrinter extends JCTree.Visitor {
 	}
 	
 	@Override public void visitCase(JCCase tree) {
-		if (tree.pat == null) {
+		// Starting with JDK12, switches allow multiple expressions per case, and can take the form of an expression (preview feature).
+		
+		List<JCExpression> pats = readObject(tree, "pats", null); // JDK 12+
+		if (pats == null) {
+			JCExpression pat = readObject(tree, "pat", null); // JDK -11
+			pats = pat == null ? List.<JCExpression>nil() : List.of(pat);
+		}
+		
+		if (pats.isEmpty()) {
 			aPrint("default");
 		} else {
 			aPrint("case ");
-			print(tree.pat);
+			print(pats, ", ");
+		}
+		
+		Enum<?> caseKind = readObject(tree, "caseKind", null); // JDK 12+
+		
+		if (caseKind != null && caseKind.name().equalsIgnoreCase("RULE")) {
+			print(" -> ");
+			if (tree.stats.head instanceof JCBreak) {
+				JCBreak b = (JCBreak) tree.stats.head;
+				print((JCExpression) readObject(b, "value", null));
+				print(";");
+				needsNewLine = true;
+				needsAlign = true;
+			} else if (tree.stats.head.getClass().getSimpleName().equals("JCYield")) {
+				print((JCExpression) readObject(tree.stats.head, "value", null));
+				print(";");
+				needsNewLine = true;
+				needsAlign = true;
+			} else {
+				print(tree.stats.head);
+				if (tree.stats.head instanceof JCBlock) needsNewLine = false;
+			}
+		} else {
+			println(": ");
+			indent++;
+			print(tree.stats, "");
+			indent--;
 		}
-		println(": ");
-		indent++;
-		print(tree.stats, "");
-		indent--;
 	}
 	
 	@Override public void visitCatch(JCCatch tree) {
@@ -1237,10 +1350,45 @@ public class PrettyPrinter extends JCTree.Visitor {
 			print(")");
 		}
 		println(" {");
-		print(tree.cases, "\n");
+		boolean ruleStyle = isCaseRuleStyle(tree.cases.head);
+		if (ruleStyle) indent++;
+		print(tree.cases, "");
+		if (ruleStyle) indent--;
 		aPrintln("}", tree);
 	}
 	
+	void printSwitchExpression(JCTree tree) {
+		aPrint("switch ");
+		JCExpression selector = readObject(tree, "selector", null);
+		if (selector instanceof JCParens) {
+			print(selector);
+		} else {
+			print("(");
+			print(selector);
+			print(")");
+		}
+		println(" {");
+		List<JCCase> cases = readObject(tree, "cases", null);
+		boolean ruleStyle = isCaseRuleStyle(cases.head);
+		if (ruleStyle) indent++;
+		print(cases, "");
+		if (ruleStyle) indent--;
+		aPrint("}");
+	}
+	
+	void printYieldExpression(JCTree tree) {
+		aPrint("yield ");
+		JCExpression value = readObject(tree, "value", null);
+		print(value);
+		println(";", tree);
+	}
+	
+	void printBindingPattern(JCTree tree) {
+		print((JCExpression) readObject(tree, "vartype", null));
+		print(" ");
+		print((Name) readObject(tree, "name", null));
+	}
+	
 	@Override public void visitTry(JCTry tree) {
 		aPrint("try ");
 		List<?> resources = readObject(tree, "resources", List.nil());
@@ -1250,9 +1398,14 @@ public class PrettyPrinter extends JCTree.Visitor {
 			break;
 		case 1:
 			print("(");
-			JCVariableDecl decl = (JCVariableDecl) resources.get(0);
-			flagMod = -1L & ~FINAL;
-			printVarDefInline(decl);
+			JCTree resource = (JCTree) resources.get(0);
+			if (resource instanceof JCVariableDecl) {
+				JCVariableDecl decl = (JCVariableDecl) resource;
+				flagMod = -1L & ~FINAL;
+				printVarDefInline(decl);
+			} else {
+				print(resource);
+			}
 			print(") ");
 			break;
 		default:
@@ -1261,8 +1414,12 @@ public class PrettyPrinter extends JCTree.Visitor {
 			int c = 0;
 			for (Object i : resources) {
 				align();
-				flagMod = -1L & ~FINAL;
-				printVarDefInline((JCVariableDecl) i);
+				if (i instanceof JCVariableDecl) {
+					flagMod = -1L & ~FINAL;
+					printVarDefInline((JCVariableDecl) i);
+				} else {
+					print((JCTree) i);
+				}
 				if (++c == len) {
 					print(") ");
 				} else {
@@ -1335,7 +1492,6 @@ public class PrettyPrinter extends JCTree.Visitor {
 	
 	static {
 		getExtendsClause = getMethod(JCClassDecl.class, "getExtendsClause", new Class<?>[0]);
-		getExtendsClause.setAccessible(true);
 		
 		if (getJavaCompilerVersion() < 8) {
 			getEndPosition = getMethod(DiagnosticPosition.class, "getEndPosition", java.util.Map.class);
@@ -1350,11 +1506,11 @@ public class PrettyPrinter extends JCTree.Visitor {
 				throw sneakyThrow(ex);
 			}
 			try {
-				storeEndMethodTemp = endPosTable.getMethod("storeEnd", JCTree.class, int.class);
+				storeEndMethodTemp = Permit.getMethod(endPosTable, "storeEnd", JCTree.class, int.class);
 			} catch (NoSuchMethodException e) {
 				try {
 					endPosTable = Class.forName("com.sun.tools.javac.parser.JavacParser$AbstractEndPosTable");
-					storeEndMethodTemp = endPosTable.getDeclaredMethod("storeEnd", JCTree.class, int.class);
+					storeEndMethodTemp = Permit.getMethod(endPosTable, "storeEnd", JCTree.class, int.class);
 				} catch (NoSuchMethodException ex) {
 					throw sneakyThrow(ex);
 				} catch (ClassNotFoundException ex) {
@@ -1363,13 +1519,13 @@ public class PrettyPrinter extends JCTree.Visitor {
 			}
 			storeEnd = storeEndMethodTemp;
 		}
-		getEndPosition.setAccessible(true);
-		storeEnd.setAccessible(true);
+		Permit.setAccessible(getEndPosition);
+		Permit.setAccessible(storeEnd);
 	}
 	
 	private static Method getMethod(Class<?> clazz, String name, Class<?>... paramTypes) {
 		try {
-			return clazz.getMethod(name, paramTypes);
+			return Permit.getMethod(clazz, name, paramTypes);
 		} catch (NoSuchMethodException e) {
 			throw sneakyThrow(e);
 		}
@@ -1379,7 +1535,7 @@ public class PrettyPrinter extends JCTree.Visitor {
 		try {
 			Class<?>[] c = new Class[paramTypes.length];
 			for (int i = 0; i < paramTypes.length; i++) c[i] = Class.forName(paramTypes[i]);
-			return clazz.getMethod(name, c);
+			return Permit.getMethod(clazz, name, c);
 		} catch (NoSuchMethodException e) {
 			throw sneakyThrow(e);
 		} catch (ClassNotFoundException e) {
@@ -1418,11 +1574,10 @@ public class PrettyPrinter extends JCTree.Visitor {
 		Field f = c.get(fieldName);
 		if (f == null) {
 			try {
-				f = tClass.getDeclaredField(fieldName);
+				f = Permit.getField(tClass, fieldName);
 			} catch (Exception e) {
 				return defaultValue;
 			}
-			f.setAccessible(true);
 			c.put(fieldName, f);
 		}
 		
@@ -1452,11 +1607,38 @@ public class PrettyPrinter extends JCTree.Visitor {
 			printAnnotatedType0(tree);
 		} else if ("JCPackageDecl".equals(simpleName)) {
 			// Starting with JDK9, this is inside the import list, but we've already printed it. Just ignore it.
+		} else if ("JCSwitchExpression".equals(simpleName)) { // Introduced as preview feature in JDK12
+			printSwitchExpression(tree);
+		} else if ("JCYield".equals(simpleName)) { // Introduced as preview feature in JDK13, part of switch expressions.
+			printYieldExpression(tree);
+		} else if ("JCBindingPattern".equals(simpleName)) { // Introduced as preview in JDK14
+			printBindingPattern(tree);
 		} else {
 			throw new AssertionError("Unhandled tree type: " + tree.getClass() + ": " + tree);
 		}
 	}
 	
+	private boolean isCaseRuleStyle(JCCase tree) {
+		if (tree == null) return false;
+		Enum<?> caseKind = readObject(tree, "caseKind", null); // JDK 12+
+		return caseKind != null && caseKind.name().equalsIgnoreCase("RULE");
+	}
+	
+	private boolean jcAnnotatedTypeInit = false;
+	private Class<?> jcAnnotatedTypeClass = null;
+	
+	private boolean isJcAnnotatedType(Object o) {
+		if (o == null) return false;
+		if (jcAnnotatedTypeInit) return jcAnnotatedTypeClass == o.getClass();
+		Class<?> c = o.getClass();
+		if (c.getSimpleName().equals("JCAnnotatedType")) {
+			jcAnnotatedTypeClass = c;
+			jcAnnotatedTypeInit = true;
+			return true;
+		}
+		return false;
+	}
+	
 	private void printMemberReference0(JCTree tree) {
 		print(readObject(tree, "expr", (JCExpression) null));
 		print("::");
@@ -1515,10 +1697,57 @@ public class PrettyPrinter extends JCTree.Visitor {
 			print(readObject(tree, "annotations", List.<JCExpression>nil()), " ");
 			print(" ");
 			print(((JCFieldAccess) underlyingType).name);
+		} else if (underlyingType instanceof JCArrayTypeTree) {
+			printTypeArray0(tree);
 		} else {
 			print(readObject(tree, "annotations", List.<JCExpression>nil()), " ");
 			print(" ");
 			print(underlyingType);
 		}
 	}
+	
+	private void printTypeArray0(JCTree tree) {
+		JCTree inner = tree;
+		int dimCount = 0;
+		
+		while (true) {
+			if (inner instanceof JCArrayTypeTree) {
+				inner = ((JCArrayTypeTree) inner).elemtype;
+				dimCount++;
+				continue;
+			} else if (isJcAnnotatedType(inner)) {
+				JCTree underlyingType = readObject(inner, "underlyingType", (JCTree) null);
+				if (underlyingType instanceof JCArrayTypeTree) {
+					inner = ((JCArrayTypeTree) underlyingType).elemtype;
+					dimCount++;
+					continue;
+				}
+			}
+			break;
+		}
+		
+		print(inner);
+		
+		inner = tree;
+		while (true) {
+			if (inner instanceof JCArrayTypeTree) {
+				dimCount--;
+				print((dimCount == 0 && innermostArrayBracketsAreVarargs) ? "..." : "[]");
+				inner = ((JCArrayTypeTree) inner).elemtype;
+				continue;
+			} else if (isJcAnnotatedType(inner)) {
+				JCTree underlyingType = readObject(inner, "underlyingType", (JCTree) null);
+				if (underlyingType instanceof JCArrayTypeTree) {
+					dimCount--;
+					print(" ");
+					print(readObject(inner, "annotations", List.<JCExpression>nil()), " ");
+					print(" ");
+					print((dimCount == 0 && innermostArrayBracketsAreVarargs) ? "..." : "[]");
+					inner = ((JCArrayTypeTree) underlyingType).elemtype;
+					continue;
+				}
+			}
+			break;
+		}
+	}
 }
